; ArcSyn3.asm  ---  TI SN94281 Sound Generator controller using Atmel AVR microcontroller
; ver 0.4.3    July 17, 2005    Alan Probandt   Portland, Oregon USA  alan_probandt@yahoo.com
; assembled with::: d:\avr\0_source\ArcadeSn\avrasm2.exe arcsyn3.asm -l arcsyn3.lst

; bugs in this version:
;
;  MIDI doesn't respond to different note presses
;
;**** The Arcade Synthesizer
;
;    This synthesizer type uses old 1980s sound generator chips to make music based on video arcade
; game sounds.  The sounds from the Texas Instruments SN74677 and SN94281 sound chips are unique
; and can't be duplicated with modern synthesizer types using FM, additive, or classical subtractive
; synthesis formats.  Originally sold inexpensively at Radio Shack, these ICs now get high prices on eBay.
;    These ICs are controlled by conbinations of resistors-capacitors on analog input pins that set
; the frequencies and by logic levels on mixer pins to set the sound type.  This program sends data to a
; digital-to-analog IC that controls the frequencies by adjusting the voltage on the analog
; input pins.  The DAC IC is itself controlled by an Atmel AVR microcontroller.  The AVR also controls
; the sound type by changing the logic levels on the sound generator IC's mixer pins.
;    The AVR microcontroller responds to ACSII commands received from any RS232 serial port using
; 38.4K baud, 8 data bits, 1 stop bit, and no parity.  This is the most common PC standard.
;    This Arcade Synth controller has a MIDI input interface and will make sounds according to MIDI
; Note On/Off messages sent to it.  It will change the sound type after receiving a MIDI program change
; message with a data value between 0 and 15.
;    This program was written for the SN94281, but can be easily adapted to the SN74677.
;    Any AVR that has one 8-bit output port and a UART can be used.  This code is designed for MAX500
; quad 8-bit DAC.  If you use a different Digital-to-Analog IC, then revise SendDACvalue routine.
;
; Commands received from RS232 are- (case insensitive)
;	X 	  reset synth; turn off all sound
;	M (0-15)  mixer selection  bits 3-0 > A B C V   binary value
;	S (0-255) change frequency - put number to SLF  - DAC channel A
;	V (0-255) change frequency - put number to VCO - DAC channel B
;	N (0-255) change frequency - put number to Noise - DAC channel C
;	P (32-96) play an SLF/VCO combination;  MIDI note-on plays same number combo
;	A (0-63)  change rhythmic pattern - MixerA timer count - equal high/low periods of 1/48 second*count
;	B (0-63)  change rhythmic pa4tern - MixerB timer count - equal high/low periods of 1/48 second*count
;	C (0-63)  change rhythmic pattern - MixerC timer count - equal high/low periods of 1/48 second*count
;	D (0-63)  change rhythmic pattern - VCOselect period counter  - equal high/low periods
;	R (0-127) change number o& 1/48 second periods before changing the random VCO and SLF values
;	Q (1-4)	  Lock Mixer to logic 1 	1=mixer A  3=mixer C
;	W (1-4)	  Lock Mixer to logic 0		2=mixer B  4=mixer V
;	E (1-4)	  Unlock Mixer bit
;	U 	  Unrandomize; turn off randomizing of VCO, SLF
;	Z 	  run Arcade synth demo
;
; Examples of commands received from PC terminal program:
;	m4<enter>  	play noise only  mixA/mixB/MixC/mixVCOsel = 0100
;	s225<enter>	slow siren-like SLF
;	a24<enter>	mixer A is logic high for 24/48ths of a second, then logic low for 24/48ths second
;	x<enter>	turn off all sound
;	w1<enter>	lock Mixer A at logic low.  Only VCO(SLF), VCO(fixedV), Noise, VCO/N sounds will play


.nolist
.include "2313DEF.inc"
.list
;.device AT90S2313

;------------------------------------------------------------------------------------------
;
;	Equates and Definitions
;
;-------------------------------------------------------------------------------------------

; System clock is 3686400 Hz from a crystal.   Ti-er 0 uses a preset of 1024.  3686400/1024=3600
; 3600/48 = 75  Timer0 overflow period of 1/48 second for Mixer switching
.equ	MixA	= 7
.equ	MixB	= 6
.equ	MixC	= 5
.equ	MixV	= 4
.equ	LockStateBit	= 7  ; set=mixer does not change value   clear=mixer toggles value when MixTimer=0
.equ	LockedValueBit = 6  ; set=mixer bit set when LockStateBit set  clear=mixer bit clear when LockStateBit clr
.equ	DACMixDDR	= DDRB
.equ	DACMixPort	= PortB ; mixA=pb7  mixB=pb6  mixC=pb5  VCOselect=pb4 :: DACdata=pb2 clock=pb1  strobe=pb0
.equ	DACdata  	= 0
.equ	DACclock 	= 1
.equ	DACload  	= 2
.equ	MIDIChanDDR	= DDRD
.equ	MIDIChanPort	= PinD; set = accept all MIDI channels, clear= MIDI channel 16 [0x0f] only
.equ	ChanPin		= 6 ;2313 pin 11  portD6
.equ	MIDIinPort	= pind
.equ	MIDIrxpin	= 2	; MIDI Receive pin is PD2 (AVR 2313 pin 6 INT0)
.equ	MIDI_TCNT	= 118	; Timer0 reload value for 31250 baud
.equ	MIDI_TCNT_Adj	= 9	; number cycles from overflow until TCNT reload
.equ	Mux48TCNT_Low	= -75	; 3686400/1024/48  == system clock/ timer preset/ # of timeouts per second
.equ	Mux48TCNT_High	= -1	; 16bit timer high register = 0xff

.equ	RAMSTART	= 0x60
.equ 	BUFFER_SIZE	= 0x06
.equ	DefaultMIDIChannel = 0x00
.equ	VCODACdefault	= 0xc0
.equ	VCODACaddr	= 0
.equ	SLFDACdefault	= 0x20
.equ	SLFDACaddr	= 1
.equ	NoiseDACaddr	= 2

; crystal de0%.$ent timing const!.t3
.equ	UB38_36864	= 5
; UART BaudRate Register value for 38400 baud with a 36864 MHz


;=====================================
;
;  	REGISTER Definitions
;
;=====================================
;
;; low register use
.def	count		= r1
.def	DACaddr		= r2
.def	DACvalue	= r3
.def	rxbitcnt	= r4	; used by MIDIinput
.def	tbin		= r5	; binary result from BCD conversion
.def	InputHead	= r6
.def	MIDIprg		= r7

.def	RandomLo	= r8
.def	RandomHi	= r9

;  upper registers   r16 - r31
.def	temp		= r16
.def	irqtemp		= r17
.def	Temp2		= r18

.def	RxData		= r19	; data byte received from MIDI synth
.def	Flags		= r20
.equ	  NewCmdFlag	= 0	; set =new command in the RS232 input buffer needing to be parsed
.equ	  UpdateMxFlag	= 1	; tells main program to get a new mixer pattern sample <now>
.equ	  Randomize	= 2	; set = change VCO and SLF with each RandomCnt countdown to zero
.equ	  FixedSound	= 3	; set means no change in sound after each timer overflow

.def	MIDIreg		= r21 ; (1<< NoteFlag) | (1<<RunStatusFlag) | (1<<DataPlaceFlag) | (Channel16Flag) | (ValidDataFlag) | (NewMIDImsg)
.equ	  RunStatusFlag	= 0	; set when the last valid message was NoteON 0x9x
.equ	  DataPlaceFlag	= 1	; set when the Cmd is NoteON and next MIDI byte is the first data byte
.equ	  Channel16Flag	= 2	; MIDI channel is 16 [0x0f] if flag set, or Omni (flag clear) PD6=0>>ChFlg=1
.equ	  ValidDataFlag	= 6	; set=current MIDI message is for Arcade Synth - MIDI chan match::clear-ignore current MIDI message being transmitted
.equ	  NewMIDImsg	= 7	; signal to main program that a new MIDI message has been received

.def	ErrorReg	= r22
.equ	  TooManyInBuffChars = 0
.equ	  CharNotDigit	= 1	; when converting ASCII digits into a binary number.
.equ	  NoParamValue	= 2
.equ	  TooManyDigits	= 3
.equ	  AnyError	= 7

;***** BCD2Bin Subroutine Register Variables

.def	tbinHiTemp	= r23	;binary result
.def	fBCDL		= r23	;low digit of BCD input  - 1's value
.def	fBCDM		= r24	;mid digit of BCD  - 10's value
.def	fBCDH		= r25	;high digit /f BCD input  - 100's value

.def	MixerATimer	= r23	; bit 7 is LockStateBit   bit 6 is LockedValueBit
.def	MixerBTimer	= r24
.def	MixerCTimer	= r25
.def	MixerVTimer	= r26
.def	CurRandomCnt  = r27
.equ	MixerATimerReg	= 23	; used to index register values with ld reg,z instruction
;
;---------------------------------------------------------------------------
;
;	Static RAM initialization	DATA SEGMENT
;
;---------------------------------------------------------------------------

.dseg
.org RAMSTART

MixerAPeriod:
.byte	1
MixerBPeriod:
.byte	1
MixerCPeriod:
.byte	1
MixerVPeriod:
.byte	1
RdmPeriodCnt:
.byte	1


MIDIcmd:	;storage of the lastest MIDI command
.byte	1
MIDI1stData:
.byte	1
MIDI2ndData:
.byte 	1
SelectMIDIChannel:
.byte	1

DACA_VCO:
.byte	1
DACB_SLF:
.byte	1
DACC_Noise:
.byte	1

INPUT_BUFFER:
.byte	BUFFER_SIZE

;%%%%%%%%%%%%%%%%%%%%%%%%%%

;---------------------------------------------------------------------------
;
;	Program initialization	CODE SEGMENT
;
;---------------------------------------------------------------------------
.cseg  ; load the following code into AVR Flash Program memory

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;   Set up Interrupt Vectors for various CPU devices
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


.org $0000
; AT90S2313 interrupt table

	rjmp	reset		; Reset handler   $000
.org INT0addr
	rjmp	INT0_IRQ	; External interrupt 0 handler  $001 (pin 6)  MIDI receive start bit
.org INT1addr
	reti	; External interrupt 1 handler  $002
.org ICP1addr
	reti	; Timer1 capture event handler  $003
.org OC1addr
	reti	; Timer1 compare match     $004
.org OVF1addr
	rjmp	TIM1_OVF	; SoftUART MIDI input activated by EXT_INT0
.org OVF0addr
	rjmp	TIM0_OVF	; SoftUART MIDIout
.org URXCaddr
	rjmp	UARTRxComplete	; UART Rx Complete RS-232 input received
.org UDREaddr
	reti	; UART Data Register empty $008
.org UTXCaddr
	reti	; UART Tx Complete	$009
.org ACIaddr
	reti	; Analog comparator handler $00a

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
; end of interrupt vectors
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


;________________________________________________________________
;
;       Initiali:ation Code		RESET	PROGRAM CODE
;________________________________________________________________

Reset:
; initialize Stack
	ldi	temp, RAMEND
	out	SPL,temp	;init Stack Pointer

; initialize I/O ports
	ser	temp
	out	DACMixDDR, temp
	ldi	temp, 0b00000111 ; start with DAC clk, data, load high- VCO
	out	DACMixPort, temp
	cbi	MIDIchanDDR, ChanPin ;MIDIChanDDR is input
	sbi	MIDIchanPort, ChanPin ;MIDIChan pullup resistor on

; initialize External Edge Interrupt Register
	ldi	temp, (1<<INT0)
	out	GIFR, temp	;  clear interrupt flag
	ldi	temp, (1<<ISC01)	;  INT0 interrupt on falling edge - MIDI inpu
	out	MCUCR, temp
	ldi	temp, (1<<INT0)	;
	out	GIMSK,temp	;  enable external INT0 - General Interrupt Mask Register

; initialize timers
; AVR2313  Timer0=8 bit ,  Timer1=16-bit
	ldi	temp, (1 << TOIE1) ; set TOIE1 - Timer Overflow 1 Interrupt Enable
	out	TIMSK,temp 	; TOIE1 enable interrupt on TC1 overflow
	out	TIFR,temp	;  to clear T/C1 overflow flag

; initialize timer1 interrupt to run continously; timer0 interrupt only runs when receiving MIDI
; initialize timer1 for 1/48 second Arcade synth mixer
	ldi	Temp, Mux48TCNT_High ; load timer1 count register for 1/48 second
	out	TCNT1H, Temp
	ldi	Temp, Mux48TCNT_Low ; load timer1 count register for 1/48 second
	out	TCNT1L, Temp
	ldi	Temp, ( (1<<CS12) | (0<<CS11) | (1<<CS10) ); start timer 1 at preset sysclk/1024
	out	TCCR1B, Temp	; start SN94281 Mux pattern

; initialize RS232 UART for AVR 90x and Mega  (AVR Tiny devices don't have UARTs)
	ldi	temp,UB38_36864
	out	UBRR, temp	; UART BaudRate Register value for 38400 baud with a 36864 MHz  - 100%
	ldi	temp, (1<<RXCIE) | (1<<RXEN) | (1<<TXEN)  ; 0b10011000
	out	UCR, temp	;  RX & TX enable - and interrupt enabled for Rx Complete

; initialize watchdog   wdr (watchdog reset) instruction is in the main code loop
	ldi	temp, (1<<wde)
	out	wdtcr, temp

; initialize general interrupts
	sei

; initialize application variables
	rcall	InitializeCommandBuffer
	ldi	temp, DefaultMIDIChannel
	sts	SelectMIDIChannel, temp
	clr	Flags
	clr	MIDIreg
	clr	MIDIprg
	clr	RandomLo  ; when randomizing VCO/SLF tones, take DAC values from Flash program code
	clr	RandomHi
	sbis	MIDIChanPort, ChanPin ; - use jumper to ground on port pin for valid MIDI on chan16 only
	sbr	MIDIreg, (1<<Channel16Flag) ; set=receive MIDI on all channels (default)

	rcall	ArcadeSynthDemo  ; start with stupid sound pattern at power-up

;________________________________________________________________
;
;       Application Routines Code		MAIN  PROGRAM CODE
;________________________________________________________________

;********************************************************************
;  Main program code for Arcade Synthesizer AVR
;********************************************************************
Main:	wdr  	; prevent watchdog from causing reset

	sbrc	MIDIreg, NewMIDImsg ; check for new MIDI message
	rjmp	ParseMIDI

	sbrc	Flags, NewCmdFlag ; check for new RS232 input
	rjmp	ParseCmdBuffer

	sbrc	Flags, UpdateMxFlag ; 1/48 second passed since last mixer update
	rjmp	UpdateMixer

	rjmp	Main

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;  enter with target ASCII command letter as first character in the INPUT_BUFFER
; This is not a subroutine.  It is a branch of the main routine.
; When completed it jumps back to the Main program entry.
ParseCmdBuffer:
	ser	temp
	mov	count, temp
	cbr	Flags, (1<<NewCmdFlag)
	ldi	ZL, Input_Buffer
	clr	ZH
	ld	temp, Z	; first get command letter to Temp
	andi	temp, 0b01011111; convert to upper case  0x6x to 0x4x
	tst	temp  ; check for empty buffer => first byte in buffer is 00
	breq	PCBfinish  ; simply exit if buffer is empty  this can happen if terminal sends 0x0d and 0x0a combo

PCB10:	inc	count
	ldi	ZL, low (CmdLtrs*2)
	ldi	ZH, high (CmdLtrs*2)
	add	ZL, count
	brcc	PCB20
	inc	ZH
PCB20:	lpm
	tst	r0
	breq	PCBnotFnd
	cp	temp, r0
	breq	PCBfnd
	rjmp	PCB10

PCBfnd:; do NOT multiply the pointer address by 2 when using IJMP
	ldi	ZH, high (CmdTable)
     	ldi	ZL, low (CmdTable)
     	add	ZL, count
     	brcc	Cfnd1
     	inc	ZH
Cfnd1:	ijmp

PCBnotFnd:
	ldi	ZL, low (NotFndTxt*2)
	ldi	ZH, high (NotFndTxt*2)
CNF1:	lpm
	tst	r0
	breq	PCBFinish
	out	UDR, r0
CNF2:	sbis	USR, UDRE
	rjmp	CNF2
	adiw	ZL,1
	rjmp	CNF1
PCBFinish:
	rcall	InitializeCommandBuffer
	rjmp	Main
NotFndTxt:
.db	"Not A Command",0x0d,0x0a,0

CmdLtrs:
.db "ABCDEMNPQRSUVWXZ",0

; table jump order must match the command letter order. No repeats

CMDTable:		; Commands received from RS232 are-
	rjmp	Acmd	;	A (0-32)  change rhythmic pattern - MixerA timer count - equal high/low periods of 1/32 second*count
	rjmp	Bcmd	;	B (0-32)  change rhythmic pattern - MixerB timer count - equal high/low periods of 1/32 second*count
	rjmp	Ccmd	;	C (0-32)  change rhythmic pattern - MixerC timer count - equal high/low periods of 1/32 second*count
	rjmp	Dcmd	;	D (0-32)  change rhythmic pattern - VCOselect period counter  - equal high/low periods
	rjmp	Ecmd	;	E (1-4)	  Set Mixer to logic 1    3=mixer C
	rjmp	Mcmd	;	M (0-15)  mixer selection  bits 3-0 > A B C V   binary value
	rjmp	Ncmd	;	N (0-255) change frequency - put number to Noise - DAC channel C
	rjmp	Pcmd	;	P (32-96) play an SLF/VCO combination;  MIDI note-on plays same number combo
	rjmp	Qcmd	;	Q (1-4)	  Lock a mixer - 	 1=mixer A
	rjmp	Rcmd	;	R (0-127)  change number of Timer1 p%riods before updating random SLF/VCO
	rjmp	Scmd	;	S (0-255) change frequency - put number to SLF  - DAC channel A
	rjmp	Ucmd  	;	U 	  Unrandomize; turn off randomizing of VCO, SLF
	rjmp	Vcmd	;	V (0-255) change frequency - put number to VCO - DAC channel B
	rjmp	Wcmd	;	W (1-4)	  Lock or Unlock a mixer  2=mixer B
	rjmp	Xcmd	;	X 	  reset synth; turn off all sound
	rjmp	Zcmd	;	Z 	  run Arcade synth demo


;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ParseMIDI:  ; this is not a subroutine.  It is a branch of the main routine.  When completed it jumps back to the Main pro'ram entr9.
	;  Two MIDI messages recognized  Note On/off and Program Change
	;  enter this routine with the MIDI message completely received. Three bytes for Note On/Off and two bytes for prgChng.
	; check for noteON, if yes check  velocity (2nd data byte) for on or off. if off, then put 0xf0 (mixer=inhibt) into MixerPort.
	;  if Note on, then get note number; check range within 32 to 96; if yes, subtract 32 and use result as
	;  index into the EEPROM tables for SLF and VCO.  Send the DAC values from the EEPROM tables.
	cbr	MIDIreg, (1<<NewMIDImsg)
	lds	temp, MIDIcmd
	swap	temp
	andi	temp, 0x0f	; isolate MIDI command type from MIDI channel
	cpi	temp, 0x09
	breq	PMnewnote

; it's a program change MIDI command - only one data byte for MIDI program change
	lds	temp,MIDI1stData
	andi	temp, 0x0f	;0x00 - 0x0f isolate the new mixer value  0000nnnn
	in	temp2, DACMixport  ; port B  mixA=pb7  mixB=pb6  mixC=pb5  VCOselect=pb4 :: DACdata=pb2 clock=pb1  strobe=pb0
	andi	temp2,0x0f  ; clear away the mixer value 0000dddd   d=DAC logic control bits 3-0
	swap	temp  ; nnnn0000
	lsl	temp	; ignore vco select mixer
	mov	MIDIprg, temp ; store sound selection
	or	temp2, temp ; don't change DAC control logic in bits 3-0  nnnndddd
	out	DACMixPort, temp2 ; put new program change value into mixer array:A=pb7 B=pb6 C=pb5  VCOsel=pb4
; Upon receipt of a MIDI message, stop playing the demo.  Stop mixer patterns.  Stop sending  random SLF and VCO values
; To resume pattern switching, send 'Z <enter>' from the terminal.
	sbr	Flags, (1<<FixedSound)
	rjmp	Main

PMnewnote:
	lds	temp,MIDI2ndData
	tst	temp  ; zero in 2nd MIDI data byte (velocity value) means turn note off
	brne	PM_PlayNewVCOnote

; turn off currently playing VCO note
	in	temp2, DACMixport
	ori	temp2, 0xf0 ; port B  mixA=pb7  mixB=pb6  mixC=pb5  VCOselect=pb4 :: DACdata=pb2 clock=pb1  strobe=pb0
	out	DACMixPort, temp2 ; put 1111 value into mixer array to inhibit all sound without changing DAC logic bits 3-0
;next data byte that is not preceeded by a cmd will be a NoteON 1st data byte, so set DP flag
	sbr	MIDIreg,  (1<<DataPlaceFlag)
	rjmp	Main

PM_PlayNewVCOnote:
	sbr	Flags, FixedSound
	lds	temp,MIDI1stData ; start playing a note on the arcade synth
	cpi	temp, 32
	brlt	PMnn3	; MIDI note value too low
	cpi	temp, 96
	brge	PMnn3	; MIDI note value too high
	subi	temp, 32
	; get the DAC values for VCO and SLF that correspond to the MIDI note number
	out 	EEAR,temp	;Set the EEPROM's address
	sbi 	EECR,EERE       ;Send the Read strobe
	in	temp2,EEDR       ;Put the data )n the transmit register
	sts	DACA_VCO, temp2
	subi	temp, -64
	out 	EEAR,temp	;Set the EEPROM's address
	sbi 	EECR,EERE       ;Send the Read strobe
	in	temp2,EEDR         ;Put the data in the transmit register
	sts	DACB_SLF, temp2
	lds	DACvalue, DACA_VCO
	ldi	temp, 0 ; VCO DAC channel
	mov	DACaddr, temp
	rcall	sendDACvalue
	lds	DACvalue, DACB_SLF
	ldi	temp, 1 ; SLF DAC channel
	mov	DACaddr, temp
	rcall	sendDACvalue
	in	temp2, DACMixport
	andi	temp2, 0x0f ; port B  mixA=pb7  mixB=pb6  mixC=pb5  VCOselect=pb4 :: DACdata=pb2 clock=pb1  strobe=pb0
;	or	temp2, MIDIprg
	out	DACMixPort, temp2 ; put 0000 value into mixer array to play VCO
; next data byte that is not preceeded by a cmd will be a NoteON 1st data byte, so set DP flag
	sbr	MIDIreg,  (1<<DataPlaceFlag)
PMnn3:	rjmp	Main



;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; This is not a subroutine.  It is a branch of the main routine.  When completed it jumps back to the Main program entry point..
;  Update Mixers  --- run this routine after every MixTimer overflow - 48 times a second
;  main program jumps here when UpdateMxFlag is set by MixTmr overflow

UpdateMixer:
	cbr	Flags, (1<<UpdateMxFlag)
	sbrc	Flags, FixedSound  ; if FixedSound flag is set, do nothing  - mixer is not changing
	rjmp	Main

;----------------------------------
; randomize VCO and SLF values
	sbrs	Flags, Randomize  ; if Randomize flag is set, send new values for SLF and VCO
	rjmp	UM_MixerA

	dec	CurRandomCnt
	brne	UM_MixerA
	lds	temp, RdmPeriodCnt ; reload countdown value
	mov	CurRandomCnt, temp

	mov	ZL, RandomLo
	mov	ZH, RandomHi
	lpm
	mov	DACvalue, R0
	ldi	temp, VCODACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue; send pseudo-random values to VCO and SLF
	adiw	ZL,1
	mov	RandomLo, ZL
	mov	RandomHi, ZH
	lpm
	mov	DACvalue, R0
	ldi	temp, SLFDACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue
	cpi	ZH, 0x03
	brlo	UM_MixerA
	clr	RandomHi

; mixer switching done with virtual-555 timers:  bit7 set=don't change mixer   -  bits5-0=current counter
; check if mixer is locked.  If yes, then reload the LockedValue
UM_MixerA:
	sbrc	MixerATimer, LockStateBit
	rjmp	UM_Lock_A_Update
	mov	temp, MixerATimer
	andi	temp, 0b00111111
	dec	temp
	andi	MixerATimer, 0b11000000
	or	MixerATimer, temp
	tst	temp
	brne	UM_MixerB
	lds	MixerATimer, MixerAPeriod
	ldi	temp, (1<<MixA)
	in	temp2, DACMixPort
	eor	temp2, temp
	out	DACMixPort, temp2
	sbrs	Flags, Randomize
	rjmp	UM_MixerB

UM_Lock_A_Update:
	in	temp2, DACMixPort
	cbr	temp2, (1<<MixA)
	sbrc	MixerATimer, LockedValueBit
	sbr	temp2, (1<<MixA)
	out	DACMixPort, temp2

UM_MixerB:
	sbrc	MixerBTimer, LockStateBit
	rjmp	UM_Lock_B_Update
	mov	temp, MixerBTimer
	andi	temp, 0b00111111
	dec	temp
	andi	MixerBTimer, 0b11000000
	or	MixerBTimer, temp
	tst	temp
	brne	UM_MixerC
	lds	MixerBTimer, MixerBPeriod
	ldi	temp, (1<<MixB)
	in	temp2, DACMixPort
	eor	temp2, temp
	out	DACMixPort, temp2
	rjmp	UM_MixerC

UM_Lock_B_Update:
	in	temp2, DACMixPort
	cbr	temp2, (1<<MixB)
	sbrc	MixerBTimer, LockedValueBit
	sbr	temp2, (1<<MixB)
	out	DACMixPort, temp2

UM_MixerC:
	sbrc	MixerCTimer, LockStateBit
	rjmp	UM_Lock_C_Update
	mov	temp, MixerCTimer
	andi	temp, 0b00111111
	dec	temp
	andi	MixerCTimer, 0b11000000
	or	MixerCTimer, temp
	tst	temp
	brne	UM_MixerV
	lds	MixerCTimer, MixerCPeriod
	ldi	temp, (1<<MixC)
	in	temp2, DACMixPort
	eor	temp2, temp
	out	DACMixPort, temp2
	rjmp	UM_MixerV

UM_Lock_C_Update:
	in	temp2, DACMixPort
	cbr	temp2, (1<<MixC)
	sbrc	MixerCTimer, LockedValueBit
	sbr	temp2, (1<<MixC)
	out	DACMixPort, temp2

UM_MixerV:
	sbrc	MixerVTimer, LockStateBit
	rjmp	UM_Lock_V_Update
	mov	temp, MixerVTimer
	andi	temp, 0b00111111
	dec	temp
	andi	MixerVTimer, 0b11000000
	or	MixerVTimer, temp
	tst	temp
	brne	UM_exit
	lds	MixerVTimer, MixerVPeriod
	ldi	temp, (1<<MixV)
	in	temp2, DACMixPort
	eor	temp2, temp
	out	DACMixPort, temp2
	rjmp	UM_exit

UM_Lock_V_Update:
	in	temp2, DACMixPort
	cbr	temp2, (1<<MixV)
	sbrc	MixerVTimer, LockedValueBit
	sbr	temp2, (1<<MixV)
	out	DACMixPort, temp2

UM_exit:  rjmp	Main
;***************************************************************************


; ===================
; subroutines
; ===================

InitializeCommandBuffer:  ; fill the input buffer with byte 00
	ldi	temp,0
	ldi	ZL,INPUT_BUFFER + BUFFER_SIZE
	clr	ZH
ICB0:	st	Z,temp
	dec	ZL
	cpi	ZL,INPUT_BUFFER-1
	brne	ICB0
	clr	InputHead
	cbr	Flags, (1<<NewCmdFlag)
	ret

GetParamValue:
; move a counter from the beginning of the digits in the input buffer until first zero 0x00 is read.
; then decrement the counter.  Counter now points a least sig digit.
;  Each digit is then  packed into registers and converted to a binary value

	clr	ErrorReg
	clr	count
	lds	temp, INPUT_BUFFER+1 ; point to first digit.  There will be at least one
	tst	temp
	brne	GPV10
	rjmp	GPVMissingDigit
GPV10:	rcall	ValidateDigit
	sbrc	ErrorReg, AnyError
	rjmp	GPV_exit

	inc	count
	lds	temp, INPUT_BUFFER+2
	tst	temp
	breq	FormatBCD
	rcall	ValidateDigit
	sbrc	ErrorReg, AnyError
	rjmp	GPV_exit

	inc	count
	lds	temp, INPUT_BUFFER+3
	tst	temp
	breq	FormatBCD
	rcall	ValidateDigit
	sbrc	ErrorReg, AnyError
	rjmp	GPV_exit

	inc	count
	lds	temp, INPUT_BUFFER+4
	tst	temp
	breq	FormatBCD
	sbr	ErrorReg, (1<<AnyError) | (1<<TooManyDigits)
	ldi	ZL, low (TooManyDigitTxt*2)
	ldi	ZH, high (TooManyDigitTxt*2)
	rcall	SendMsg

GPV_exit: ret

FormatBCD:
;enter here with count=number of valid digits and offset of least digit from INPUT_BUFFER
	push	fBCDL
	push	fBCDM
	push	fBCDH
	clr	fBCDL
	clr	fBCDM
	clr	fBCDH
	ldi	ZL, low (INPUT_BUFFER)
	clr	ZH
	add	ZL, count
	ld	fBCDL, z
	andi	fBCDL, 0x0f
	dec	count
	breq	ConvertBCD
	ldi	ZL, low (INPUT_BUFFER)
	clr	ZH
	add	ZL, count
	ld	fBCDM, z
	andi	fBCDM, 0x0f
	dec	count
	breq	ConvertBCD
	ldi	ZL, low (INPUT_BUFFER)
	clr	ZH
	add	ZL, count
	ld	fBCDH, z
	andi	fBCDH, 0x0f
	dec	count
	breq	ConvertBCD
ConvertBCD:
	rcall	BCD2bin8
	pop	fBCDH
	pop	fBCDM
	pop	fBCDL
	ret

ValidateDigit:
	mov	Temp2, temp
	swap	Temp
	andi	temp, 0b00000011  ; validate digit character in buffer
	cpi	temp, 0x03
	brne	GPVnotdigit
	andi	Temp2, 0b00001111
	cpi	Temp2, 0x0a
	brge	GPVnotdigit
	clr	ErrorReg
	ret

GPVMissingDigit:  ; set error flags and send message for no digit after command letter
	sbr	ErrorReg, (1<<AnyError) | (1<<NoParamValue)
	ldi	ZL, low (NoParamTxt*2)
	ldi	ZH, high (NoParamTxt*2)
	rcall	SendMsg
	ret

GPVnotdigit: ; set error flags and send message for char after command letter is not a digit
	sbr	ErrorReg, (1<<AnyError) | (1<<CharNotDigit)
	ldi	ZL, low (NotDigitTxt*2)
	ldi	ZH, high (NotDigitTxt*2)
	rcall	SendMsg
	ret

NoParamTxt:
.db	"No value after cmd", 0x0d, 0x0a, 0
TooManyDigitTxt:
.db	"Too many digits", 0x0d, 0x0a, 0
NotDigitTxt:
.db	"Not a digit", 0x0d, 0x0a, 0
;***************************************************************************
;*
;* "BCD2bin8" - BCD to 8-bit binary conversion
;*
;* This subroutine converts a 2-digit BCD number (fBCDH:fBCDL) to an
;* 8-bit number (tbin).
;*						   original regs
;* Number of words	:4 + return		   tbinHiTemp =r23
;* Number of cycles	:3/48 (Min/Max) + return   fBCDL =r23
;* Low registers used	:None                      fBCDM =r24
;* High registers used  :2 (tbin/fBCDL,fBCDH)      fBCDH =r25
;*
;* Modifications to make the routine accept a packed BCD number is indicated
;* as comments in the code. If the modifications are used, fBCDH shall be
;* loaded with the BCD number to convert prior to c!lling the routine.
;*
;***************************************************************************

BCD2bin8:

BCDb8_0: subi	fBCDH,1		;fBCDH = fBCDH - 1
	brcs	BCDb8_1		;if carry not set
	subi	tbinHiTemp,-100	; result = result + 10
	rjmp	BCDb8_0		; loop again
BCDb8_1: subi	fBCDM,1
	brcs	BCDb8_2
	subi	tbinHiTemp, -10
	brcs	BCDb8_1
BCDb8_2:
	mov	tbin, tbinHiTemp	; store in low bank register
	ret			;else return



;*******************************************************************
; ArcadeSynthDemo  -- have the ArcadeSynth play a pattern at power-up
;*******************************************************************

ArcadeSynthDemo:
	sbr	Flags, (1 << Randomize)
	cbr	Flags, (1 << FixedSound) ; play changing sounds

	ldi	temp, VCODACdefault  ; send initial tones to Arcade Synth
	mov	DACvalue, temp
	ldi	temp, VCODACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue

	ldi	temp, SLFDACdefault
	mov	DACvalue, temp
	ldi	temp, SLFDACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue

	clr	DACvalue
	ldi	temp, NoiseDACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue

; initialize virtual timers for mixers= fast switching with these values   LockStateBit=0  L/ckedValueBit=0
	ldi	temp, 0x0f
	sts	MixerAPeriod, temp
	mov	MixerATimer, temp
	ldi	temp, 8 ;0x16
	sts	MixerBPeriod, temp
	mov	MixerBTimer, temp
	ldi	temp, 13 ;0x0d
	sts	MixerCPeriod, temp
	mov	MixerCTimer, temp
	ldi	temp, 30 ; 0x1e
	sts	MixerVPeriod, temp
	mov	MixerVTimer, temp

	ldi	temp, 36
	sts	RdmPeriodCnt, temp
	mov	CurRandomCnt, temp

	ret


;-------------------------------------------------------------------------------------------
;SendDACvalue   Transmits value to selected MAX500 DAC channel.
;  receives- DACvalue, DACaddr  -- start with bits DACData, DACclock, and DACload at logic high
SendDACvalue:
	sbrs	DACaddr, 1	; first send channel number to MAX500
	rjmp	SDv10
	sbi	DACMixPort, DACdata
	rjmp	SDv20
SDv10:	cbi	DACMixPort, DACdata
SDv20:	cbi	DACMixPort, DACclock
	sbi	DACMixPort, DACclock
	sbrs	DACaddr, 0
	rjmp	SDv30
	sbi	DACMixPort, DACdata
	rjmp	SDv40
SDv30:	cbi	DACMixPort, DACdata
SDv40:	cbi	DACMixPort, DACclock
	sbi	DACMixPort, DACclock
; serial transmit the data value
	ldi	Temp, 8
SDv50:	lsl	DACvalue ; send MSB first
	brcc	SDv60
	sbi	DACMixPort, DACdata
	rjmp	SDv70
SDv60:	cbi	DACMixPort, DACdata
SDv70:	cbi	DACMixPort, DACclock
	sbi	DACMixPort, DACclock
	dec	Temp
	brne	SDv50
	cbi	DACMixPort, DACload
	sbi	DACMixPort, DACload
	ret

;-------------------------------------

SendMsg:  ; enter with Zh,Zl pointing to zero-terminated text string
	lpm
	tst	r0
	brne	smg1
	ret
smg1:	out	UDR, r0
smg2:	sbis	USR, UDRE
	rjmp	smg2
	adiw	ZL, 1
	rjmp	SendMsg

;-------------------------------------
;  Command code
;-------------------------------------

Xcmd:	; reset Arcade Synth and switch off sound
	sbr	Flags, (1<<FixedSound)
	ldi	temp, 0xf7
	out	DACMixPort, temp
	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------

Mcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Mcmd1
	sbr	Flags, (1<<FixedSound)
	mov	temp, tbin
	andi	temp, 0x0f
	swap	temp
; port B  mixA=pb7  mixB=pb6  mixC=pb5  VCOselect=pb4 :: DACdata=pb2 clock=pb1  strobe=pb0
	in	temp2,DACMixPort
	andi	temp2, 0b00001111 ; clear the mixer selection only
	or	temp2, temp ; set the new mixer selection
	out	DACMixPort, temp2

Mcmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------

Scmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Scmd1
	sts	DACB_SLF, tbin
	mov	DACvalue, tbin
	ldi	temp,SLFDACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue

Scmd1:	rcall	InitializeCommandBuffer
	rjmp	Main


;-------------------------------------

Vcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Vcmd1
	sts	DACA_VCO, tbin
	mov	DACvalue, tbin
	ldi	temp, VCODACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue

Vcmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;------------------------------------

Ncmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Ncmd1
	sts	DACC_Noise, tbin
	mov	DACvalue, tbin
	ldi	temp,  NoiseDACaddr
	mov	DACaddr, temp
	rcall	SendDACvalue
Ncmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------

Pcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Pcmd1
	; not implemented yet
Pcmd1:	rcall	InitializeCommandBuffer
	rjmp	Main


;-------------------------------------
; Change mixer A virtual-555 timer period.
Acmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Acmd1

	lds	temp, MixerAPeriod
	andi	temp, (1<<LockStateBit) | (1<<LockedValueBit) ;0b11000000 ; preserve the status of the LockStateBit
	mov	temp2, tbin
	andi	temp2, 0b00111111 ; max new value is 63  0x3f
	or	temp, temp2
	sts	MixerAPeriod, temp

Acmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Change mixer B virtual-555 timer period.
Bcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Bcmd1

	lds	temp, MixerBPeriod
	andi	temp, (1<<LockStateBit) | (1<<LockedValueBit) ;0b11000000 ; preserve the status of the LockStateBit
	mov	temp2, tbin
	andi	temp2, 0b00111111 ; max new value is 63  0x3f
	or	temp, temp2
	sts	MixerBPeriod, temp

Bcmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Change mixer C virtual-555 timer period.
Ccmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Ccmd1
	lds	temp, MixerCPeriod
	andi	temp, (1<<LockStateBit) | (1<<LockedValueBit) ;0b11000000 ; preserve the status of the LockStateBit
	mov	temp2, tbin
	andi	temp2, 0b00111111 ; max new value is 63  0x3f
	or	temp, temp2
	sts	MixerCPeriod, temp

Ccmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Change mixer V virtual-555 timer period.
Dcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Dcmd1
	lds	temp, MixerVPeriod
	andi	temp, (1<<LockStateBit) | (1<<LockedValueBit) ;0b11000000 ; preserve the status of the LockStateBit
	mov	temp2, tbin
	andi	temp2, 0b00111111 ; max new value is 63  0x3f
	or	temp, temp2
	sts	MixerVPeriod, temp

Dcmd1:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Change number of timer1 periods before changing random SLF/VCO values.
Rcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Rcmd100
	mov	temp, tbin
	tst	tbin
	brne	Rcmd10
	cbr	Flags, (1<< Randomize)  ; zero value means turn off randomization of SLF/VCO
	rjmp	Rcmd100

Rcmd10:	brmi	Rcmd100 ; new random countdown value is out of range (0-127)
	sbr	Flags, (1<< Randomize)
	sts	RdmPeriodCnt, tbin

Rcmd100: rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Qcmd (1-4)    Lock bit at logic 1 --    LockStateBit and LockedValueBit are both set

Qcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Qcmd10
	mov	temp, tbin
	cpi	temp, 5
	brsh	Qcmd10	; skip processing if parameter is more than 4

	ldi	ZL, low (MixerAPeriod)  ;r23 = MixerATimer	0x60 = MixerAPeriod
	dec	ZL    ; 0x5f            ;r24 = MixerBTimer      0x61 = MixerBPeriod
	clr	ZH                      ;r25 = MixerCTimer      0x62 = MixerCPeriod
	add	ZL, tbin                ;r26 = MixerVTimer      0x63 = MixerVPeriod
	ld	temp, z
	ori	temp,0b11000000 ; set lock mixer bit in SRAM
	st	z, temp
	ldi	ZL, MixerATimerReg-1; 22
	add	ZL, tbin
	ld	temp, z
	ori	temp,0b11000000 ; set lock mixer bit in register
	st	z, temp

Qcmd10:	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Wcmd (1-4)    Lock bit at logic 0 --    LockStateBit is set.  LockedValueBit is Cleared.

Wcmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Wcmd10
	mov	temp, tbin
	cpi	temp, 5
	brsh	Wcmd10	; skip processing if parameter is more than 4

	ldi	ZL, low (MixerAPeriod) ;r23 = MixerATimer	0x60 = MixerAPeriod
	dec	ZL                     ;r24 = MixerBTimer      0x61 = MixerBPeriod
	clr	ZH                     ;r25 = MixerCTimer      0x62 = MixerCPeriod
	add	ZL, tbin               ;r26 = MixerVTimer      0x63 = MixerVPeriod
	ld	temp, z
	andi	temp, 0b00111111 ;clear LockStateBit and LockedValueBit in SRAM
	ori	temp, 0b10000000 ;set LockStateBit in SRAM
	st	z, temp
	ldi	ZL, MixerATimerReg-1
	add	ZL, tbin
	ld	temp, z
	andi	temp, 0b00111111 ;clear LockStateB in register
	ori	temp, 0b10000000 ;set LockStateBit
	st	z, temp

Wcmd10: rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
; Ecmd (1-4)    Unlock mixer bit

Ecmd:	rcall	GetParamValue
	sbrc	ErrorReg, AnyError
	rjmp	Ecmd10
	mov	temp, tbin
	cpi	temp, 5
	brsh	Ecmd10	; skip processing if parameter is more than 4 [brsh - unsigned]

	ldi	ZL, low (MixerAPeriod)  ;r23 = MixerATimer	0x60 = MixerAPeriod
	dec	ZL                      ;r24 = MixerBTimer      0x61 = MixerBPeriod
	clr	ZH                      ;r25 = MixerCTimer      0x62 = MixerCPeriod
	add	ZL, tbin                ;r26 = MixerVTimer      0x63 = MixerVPeriod
	ld	temp, z
	andi	temp,0b01111111 ; clear LockStateBit 7  in SRAM
	st	z, temp
	ldi	ZL, MixerATimerReg-1
	add	ZL, tbin
	ld	temp, z
	andi	temp,0b01111111 ; clear LockStateBit 7  in register
	st	z, temp

Ecmd10:	rcall	InitializeCommandBuffer
	rjmp	Main


;------------------------------------------
Ucmd:	; toggle randomize flag
	ldi	temp, (1<<Randomize)
	eor	Flags, temp
	rcall	InitializeCommandBuffer
	rjmp	Main

;-------------------------------------
Zcmd: ; reset and run demo program
	clr	Flags
	clr	MIDIReg
	clr	ErrorReg
	clr	RandomHi
	clr	RandomLo
	rcall	InitializeCommandBuffer
	rcall	ArcadeSynthDemo
    	rjmp	Main


;____________________________________________________________________
;
;  Interrupt Routine code for Arcade Synthesizer AVR
;____________________________________________________________________


;***************************************************************************
; UARTRxComplete  - interrupt happens when byte is finished being received
;			by the UART from the PC by RS232 at 38.4K baud.
;***************************************************************************

UARTRxComplete:
	push	irqtemp
	push	ZH
	push	ZL

	in	irqtemp, UDR
	cpi	irqtemp, 0x0d
	breq	URxC10
	cpi	irqtemp, 0x0a
	breq	URxC10

	ldi	ZL, low(INPUT_BUFFER)	;
	clr	ZH			;
	add	ZL, InputHead		;  ADD can use all registers
	in	irqtemp,UDR 		;   new char from PC into interrupt temp register
	st	Z,irqtemp		;
	inc	InputHead		;  INC can use all registers
	mov	irqtemp,InputHead
	cpi	irqtemp,BUFFER_SIZE	;  CPI must use upper registers
	brne	URxCout			; usually

	ldi	irqtemp,  (1<< AnyError) | (1<<TooManyInBuffChars)
	mov	errorReg, irqtemp

	clr	InputHead
	rjmp	URxCout

URxC10:	ldi	ZL, low (INPUT_BUFFER) ; check for a letter in first buffer position before setting NewCmdFlag
	clr	ZH
	ld	irqtemp, z
	cpi	irqtemp, 0x41	;capital A
	brlo	URxCout
	sbr	Flags, (1<<NewCmdFlag)

URxCout:
	pop	ZL
	pop	ZH
	pop	irqtemp
	reti			; 4 cycles


;**************************************************************************
;  Receive MIDI -- Interrupt controlled for devices with no UART
;
;  Two parts - a setup subroutine and a timer interrupt routine.
;  When the MIDI transmission from the synth starts, the falling
;	edge of the start bit causes an interrupt on INT0 (pin 6)
;  Each MIDI byte begins being received with a call to this IRQ.
;
;  After the falling edge of the start bit, the INT0 IRQ is disabled
;	until the entire MIDI byte (excluding the stop bit) has assembled.
;  Then the INT0 routine is enabled to catch the falling edge of the
;	start bit for the next byte sent by the MIDI synthesizer.
;  If the INT0 interrupt flag is not cleared (GIFR bit 6 set!) then the
;   	entire MIDI-receive process restarts with a 0xFF byte.
;***************************************************************************
INT0_IRQ:  ; ; 6 cycles interrupt latency to start here
; disable External0 Interrupt

	ldi	irqtemp, (1<<INT0)
	out	GIFR, irqtemp   ; clear interrupt flag
	clr	irqtemp		; 1 cycle
	out	GIMSK,irqtemp	; 1 cycle  - disable external INT0 - General Inter Mask Reg (bit 6)

	ldi	irqtemp,8
	mov	rxBitCnt,irqtemp; 1 cycle
	ser	RxData		; 1 cycle

; enable timer0 interrupt on overflow without changing timer 1 irq
	in	irqtemp, TIMSK
	ori	irqtemp,1 << TOIE0 ; 1 cycle  - set TOIE1 - Timer Overflow 1 Interrupt Enable
	out	TIMSK,irqtemp 	; 1 cycle TOIE1 enable interrupt on TC1 overflow
	ldi	irqtemp, (1 << TOV0)
	out	TIFR,irqtemp	; 1 cycle  to clear T/C0 overflow flag
; set timer0 overflow value
	ldi	irqtemp, (~(MIDI_TCNT+(MIDI_TCNT/2))) + MIDI_TCNT_adj ;  start bit and one/half
	out	TCNT0, irqtemp 	; 1 cycle

; start timer0 running
	ldi	irqtemp,1<<CS10	; 1 cycle
	out	TCCR0,irqtemp 	; 1 cycle - start running timer1 at system clock
	reti			; 4 cycles


;*************************************************************************
;
;   TIM0_OVF - MIDI input activated by INT0
;	Parses MIDI input into MIDI message when new byte is fully received
;
;INT0irq --|xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--|xxxxxxxxxxx
;T0irq-----------|----|---|----|---|----|---|----|----|-----------|--
;data------\___/----X___X----X___X----X___X----X___/-----\____/------
;Rxbitcnt        |8   |7  |6   |5  |4   |3  |2   |1   |0
;Rx bit   start  0    1   2    3   4    5   6    7   stop  start+1
;*************************************************************************

TIM0_OVF:   		; 6 cycles interrupt latency to start IRQ code
; reload count register and assemble arriving bits into a MIDI byte
	ldi	irqtemp, (-(MIDI_TCNT)) + MIDI_TCNT_adj
	out	TCNT0, irqtemp
 	tst	rxBitCnt	; if bitcnt=0 then byte reception completed; parse the received byte
 	breq	MRx_ParseMIDI
	sec
	sbis	MIDIinPort, MIDIrxpin 	; read data from pin
	clc
	ror	RxData  	; 0 > c7654321 rotate carry into data register
	dec 	rxBitCnt
	rjmp	MRxOut

MRx_ParseMIDI:
; test if new MIDI data is a command or data byte
	tst	RxData
	brmi	MRxNewCmd 	; MIDI command byte received - bit 7 set

; If the Arcade Synth is receiving only on one MIDI channel, (PD6 = gnd)
; and the previous MIDI command byte was not that channel, then ignore all incoming data bytes
; until a new MIDI command byte is received.  Process incoming MIDI data bytes only if the ValidDataFlag is set.

	sbrs	MIDIreg, ValidDataFlag	; if data byte, check if channel is valid for parsing
	rjmp	MRxFinish	; not valid channel, so ignore received data byte

; MIDI data that just arrived is valid. Check if it is part of a NoteOn msg or a Prg Change msg
	sbrs	MIDIreg, RunStatusFlag; previous MIDI message was a NoteON. If this message..
	rjmp	MRxNewPrgData 	 ; is also a NoteON, then it does not need a command byte.

	; this is data for a NoteON
	; DataPlaceFlag status validated by MRxNewCmd
	sbrs	MIDIreg, DataPlaceFlag;  is it the first or second data byte on NoteON?
	rjmp	MRxVelocityData  ;  2nd data byte Velocity
	;first MIDI NoteO data byte  -- MIDI note number  store byte and update flag status
	cbr	MIDIreg, (1<<DataPlaceFlag) ; validate flag for 2nd data byte
	sts	MIDI1stData, RxData ; store the MIDI data byte that just arrived in SRAM MIDI message area
	rjmp	MRxFinish

MRxVelocityData:  ; 5pdate flags and store arriving MIDI data byte
	sbr	MIDIreg, (1<<NewMIDImsg) ; flag to tell main program that a new MIDI message received
	sts	MIDI2ndData, RxData
	;  RunStatusFlag and ValidDataFlag remain set
	cbr	MIDIreg,  (1<<DataPlaceFlag);  RunStatusFlag remains set
	rjmp	MRxFinish

MRxNewPrgData:  ; update flags and store arriving MIDI data byte
	cbr	MIDIreg,  (1<<DataPlaceFlag) | (1<<RunStatusFlag) | (1<<ValidDataFlag)
	sts	MIDI1stData, RxData ; store the MIDI data byte that just arrived in SRAM MIDI message area
	sbr	MIDIreg, (1<<NewMIDImsg) ; flag to tell main program that a new MIDI message received
	rjmp	MRxFinish

MRxNewCmd:
	mov	irqtemp, RxData  ; check if new MIDI cmd is for the Arcade Synth MIDI channel
	sbrs	MIDIreg, Channel16Flag
	rjmp	MRxOmni  ; Chan16Flag clear means accept all MIDI channels
	andi	irqtemp, 0b00001111  ; ignore all but channel 16 check if new command is on correct channel
	lds	Temp2, SelectMIDIChannel
	cp	irqtemp, temp2 ; check if correct channel
	brne	MRxIgnore
MRxOmni:
	sts	MIDIcmd, RxData	; yes, selected MIDI channel for Arcade Synth
	andi	irqtemp, 0xf0  ; determine MIDI command type
	swap	irqtemp		; 0x0 0 0 0 MIDI cmd type
	cpi	irqtemp, 0x09	; Note On command
	brne	MRxNewPrgCmd	; branch if not a NoteON command
	; yes, it is a NoteON command
	sbr	MIDIreg, (1<<RunStatusFlag) | (1<<DataPlaceFlag) | (1<<ValidDataFlag)
	rjmp	MRxFinish
MRxNewPrgCmd:
	cpi	irqtemp, 0x0c	;  is it a Program Change command
	brne	MRxIgnore	; not a PrgChg cmd, so ignore [clr Valid flag]
	; yes. it is a new PrgChg command
	cbr	MIDIreg, (1<<RunStatusFlag) | (1<<DataPlaceFlag)
	sbr	MIDIreg, (1<<ValidDataFlag)
	rjmp	MRxFinish
MRxIgnore:
	cbr	MIDIreg, (1<<ValidDataFlag) | (1<<RunStatusFlag) | (1<<DataPlaceFlag)

MRxFinish:  ; turn off timer 0 ;re-enable the INT0 interrupt to catch next MIDI in start bit
	out 	TCCR0, irqtemp 	;  turn off timer counter0
	ldi	irqtemp, 1<<INT0
	out	GIFR, irqtemp  	;  clear INT0 flag (if still set) by writing high * important to clear this flag!!
	out	GIMSK, irqtemp	;  enable INT0 in General Interrupt Mask Register
	ldi	irqtemp, 1<<ISC01 ;  set the interrupt to happen on the falling edge of the start bit
	out	MCUCR, irqtemp

MRxOut:	reti


;***********************************************************************************************************
;
; Timer 1 Overflow Interrupt Code  - Arcade Synth mixer
;
; Overflows every 1/48 of a second.  Reloads timer, sets the UpdateMixer flag and exits.
;
;************************************************************************************************************

TIM1_OVF:  ; 16-bit timer on AVR2313 and AVR4433, 8-bit with sys/16384 preset on Tiny26
	ldi	irqtemp, Mux48TCNT_High
	out	TCNT1H,irqtemp  ;   put timeout count in timer
	ldi	irqtemp, Mux48TCNT_Low
	out	TCNT1L,irqtemp  ; put timeout count in timer

	sbr	Flags, (1<<UpdateMxFlag) ; tell main program to read the mixer pattern

	reti

;;====================================================

; ********************** ASCII strings
;	"****  ArcSyn3.asm  ver 0.4.2  Jul 17, 2005  alan_probandt@yahoo.com "
.db     "****  rAScny.3sa mv re0 4.2.  uJ l71 ,0250  lanap_orabdn@tayoh.ooc m"

.eseg  ; VCO and SLF settings used to make notes from MIDI messages
.db  254, 250, 246, 242, 238, 234, 230, 226
.db  222, 218, 214, 210, 206, 202, 198, 194
.db  190, 186, 182, 178, 174, 170, 166, 162
.db  158, 154, 150, 146, 142, 138, 134, 130
.db  126, 122, 118, 114, 110, 106, 102,  98
.db   98,  94,  90,  86,  82,  78,  74,  70
.db   66,  62,  58,  54,  50,  46,  42,  38
.db   34,  30,  26,  22,  18,  14,  10,   6

.db   34,  30,  26,  22,  18,  14,  10,   6
.db   66,  62,  58,  54,  50,  46,  42,  38
.db   98,  94,  90,  86,  82,  78,  74,  70
.db  126, 122, 118, 114, 110, 106, 102,  98
.db  158, 154, 150, 146, 142, 138, 134, 130
.db  190, 186, 182, 178, 174, 170, 166, 162
.db  222, 218, 214, 210, 206, 202, 198, 194
.db  254, 250, 246, 242, 238, 234, 230, 226




;
;		Atmel AVR AT90s2313 [Timer0-8bit Timer1-16bit]
;			RESET\	|1   20	| VCC +5V
;  from MAX232 pin 9	PD0 RxD	|2   19	| PB7 MixerA pin 16 on SN94281 - SCK
;  to MAX232 pin 10	PD1 TxD	|3   18	| PB6 MixerB pin 15 on SN94281 - MISO
;  	 3.6864 MHz 	XTAL1  	|4   17	| PB5 MixerC pin 14 on SN94281 - MOSI
;  	  	  	XTAL2  	|5   16	| PB4 MixerV pin 13 on SN94281
;  	MIDI in	INT0	PD2 	|6   15	| PB3
;			PD3 	|7   14	| PB2	DACload
;       		PD4  	|8   13	| PB1  	DACclock
;   			PD5  	|9   12	| PB0  	DACdata
;     			GROUND	|10  11	| PD6   MIDI channel select; 0=omni, 1=channel 16
;
;	         MAX500
;	        __________
;  SLF	  OUTB  |1     16| OUTC  Noise
;  VCO	  OUTA  |2     15| OUTD  Volume?
;	gnd VSS |3     14| VDD  +12v
;  +3.5V VREFAB |4     13| VREFC   +3.5V
;	   AGND |5     12| VREFD
;	   DGND |6     11| SRO serial data out
;	  LDAC\ |7     10| CLOCK
;	   DATA |8      9| LOAD\
;	        ---------
; LOAD\ float for two-wire interface
; VDD  +12 volts (+11.4 - +16.5) 10mA
; VREF full scale analog output (2v -- Vdd-4v)  11K ohms input resistance
; digital inputs TTL or +5v CMOS
; DAC outputs can sink/source 5 mA
; LDAC\ tied low causes outputs to reflect input DAC registers

;
;	     Texas Inst SN94281
;	        __________
;.001uF	noise C |1     16| Mixer A                Mixer A   B   C
;DAC-C	noise R |2     15| Mixer B            		0   0   0 VCO
;	volume  |3     14| Mixer C         		1   0   0 SLF
;	+5 VReg |4     13| Mux1  - VCO select           0   1   0 Noise
;      audio in |5     12| SLF capacitor   1uF         	1   1   0 VCO/Noise
;	9V+ in  |6     11| SLF resistor DAC-B  		0   0   1 SLF/Noise
;     audio out |7     10| VCO resistor DAC-A           1   0   1 SLF/VCO/Noise
;	   gnd  |8      9| VCO capacitor  .002uF        0   1   1 SLF/VCO
;	        ---------                               1   1   1 Inhibit


;	         MAX232
;	        __________
;   1uF	cap1 +	|1     16| +5V
;  Gnd -cap2+9v |2     15| GND    (DB9 pin 5)
;  	cap1 -  |3     14| rs232out T1  to PC (DB9 pin 2)
;  1uF	cap3 +  |4     13| rs232in  R1  from PC (DB9 pin 3)
; 	cap3 -  |5     12| TTL out R1  to pin 2  AVR2313  RxD
;  Gnd +1uF -9v |6     11| TTL in T1   to pin 3 AVR2313  TxD
;    rs232out T2|7     10| TTL in T2
;    rs232in R2 |8      9| TTL out R2
;	        ---------